package ru.dbotthepony.mc.otm.client.screen.panels.slot

import com.mojang.blaze3d.platform.InputConstants
import com.mojang.blaze3d.vertex.PoseStack
import net.minecraft.ChatFormatting
import net.minecraft.network.chat.Component
import net.minecraft.world.inventory.Slot
import net.minecraft.world.item.Item
import net.minecraft.world.item.ItemStack
import net.minecraft.world.item.Items
import net.minecraftforge.client.extensions.common.IClientItemExtensions
import ru.dbotthepony.mc.otm.client.isCtrlDown
import ru.dbotthepony.mc.otm.client.minecraft
import ru.dbotthepony.mc.otm.client.playGuiClickSound
import ru.dbotthepony.mc.otm.client.render.MatterySprite
import ru.dbotthepony.mc.otm.client.render.drawColor
import ru.dbotthepony.mc.otm.client.render.drawRect
import ru.dbotthepony.mc.otm.client.screen.MatteryScreen
import ru.dbotthepony.mc.otm.client.screen.panels.EditablePanel
import ru.dbotthepony.mc.otm.core.GetterSetter
import ru.dbotthepony.mc.otm.core.TextComponent
import ru.dbotthepony.mc.otm.core.TranslatableComponent
import ru.dbotthepony.mc.otm.core.math.RGBAColor
import ru.dbotthepony.mc.otm.menu.UserFilteredSlot

abstract class UserFilteredSlotPanel<out S : MatteryScreen<*>, out T : Slot>(
	screen: S,
	parent: EditablePanel<*>?,
	slot: T,
	x: Float = 0f,
	y: Float = 0f,
	width: Float = SIZE,
	height: Float = SIZE,
	noItemIcon: MatterySprite? = null
) : SlotPanel<S, T>(screen, parent, slot, x, y, width, height, noItemIcon) {
	abstract var slotFilter: Item?

	override fun renderSlotBackground(stack: PoseStack, mouseX: Float, mouseY: Float, partialTick: Float) {
		super.renderSlotBackground(stack, mouseX, mouseY, partialTick)

		if (slotFilter != null) {
			if (slotFilter !== Items.AIR) {
				val itemStack = ItemStack(slotFilter!!, 1)

				screen.renderItemStack(stack, itemStack, null)
				clearDepth(stack)

				drawColor = SLOT_FILTER_COLOR
			} else {
				drawColor = SLOT_BLOCK_COLOR
			}

			drawRect(stack, 0f, 0f, width, height)
		}
	}

	override fun innerRenderTooltips(stack: PoseStack, mouseX: Float, mouseY: Float, partialTick: Float): Boolean {
		if (isHovered && slotFilter != null && slotFilter !== Items.AIR && itemStack.isEmpty) {
			val itemstack = ItemStack(slotFilter!!, 1)

			screen.renderComponentTooltip(
				stack,
				getItemStackTooltip(itemstack).toMutableList().also {
					it.add(0, TranslatableComponent("otm.gui.slot_filter.filtered").withStyle(ChatFormatting.GRAY))
					it.add(1, TranslatableComponent("otm.gui.slot_filter.hint").withStyle(ChatFormatting.GRAY))
					it.add(2, TextComponent(""))
				},
				mouseX.toInt(),
				mouseY.toInt(),
				IClientItemExtensions.of(itemstack).getFont(itemstack, IClientItemExtensions.FontContext.TOOLTIP) ?: screen.font,
				itemstack
			)
		} else if (isHovered && slotFilter === Items.AIR && itemStack.isEmpty) {
			screen.renderComponentTooltip(
				stack,
				ArrayList<Component>().also {
					it.add(TranslatableComponent("otm.gui.slot_filter.forbidden").withStyle(ChatFormatting.GRAY))
					it.add(TranslatableComponent("otm.gui.slot_filter.hint").withStyle(ChatFormatting.GRAY))
				},
				mouseX.toInt(),
				mouseY.toInt(),
				screen.font
			)
		}

		return super.innerRenderTooltips(stack, mouseX, mouseY, partialTick)
	}

	override fun mouseClickedInner(x: Double, y: Double, button: Int): Boolean {
		if (button == InputConstants.MOUSE_BUTTON_LEFT && minecraft.window.isCtrlDown) {
			if (slotFilter === null) {
				if (screen.menu.carried.isEmpty) {
					slotFilter = slot.item.item
				} else {
					slotFilter = screen.menu.carried.item
				}
			} else {
				slotFilter = null
			}

			playGuiClickSound()

			return true
		} else {
			return super.mouseClickedInner(x, y, button)
		}
	}

	override fun mouseReleasedInner(x: Double, y: Double, button: Int): Boolean {
		if (button == InputConstants.MOUSE_BUTTON_LEFT && minecraft.window.isCtrlDown) {
			return true
		}

		return super.mouseReleasedInner(x, y, button)
	}

	companion object {
		val SLOT_FILTER_COLOR = RGBAColor(85, 113, 216, 150)
		val SLOT_BLOCK_COLOR = RGBAColor(219, 113, 113, 150)

		fun <S : MatteryScreen<*>, T : Slot> of(
			screen: S,
			parent: EditablePanel<*>?,
			slot: T,
			x: Float = 0f,
			y: Float = 0f,
			width: Float = SIZE,
			height: Float = SIZE,
			noItemIcon: MatterySprite? = null,
			filter: GetterSetter<Item?>
		): UserFilteredSlotPanel<S, T> {
			return object : UserFilteredSlotPanel<S, T>(screen, parent, slot, x, y, width, height, noItemIcon) {
				override var slotFilter: Item? by filter
			}
		}

		fun <S : MatteryScreen<*>, T : UserFilteredSlot> of(
			screen: S,
			parent: EditablePanel<*>?,
			slot: T,
			x: Float = 0f,
			y: Float = 0f,
			width: Float = SIZE,
			height: Float = SIZE,
			noItemIcon: MatterySprite? = null
		): UserFilteredSlotPanel<S, T> {
			return object : UserFilteredSlotPanel<S, T>(screen, parent, slot, x, y, width, height, noItemIcon) {
				override var slotFilter: Item?
					get() = slot.filter?.get()
					set(value) { slot.filter?.accept(value) }
			}
		}
	}
}
