package ru.dbotthepony.mc.otm.item.exopack

import net.minecraft.ChatFormatting
import net.minecraft.network.chat.Component
import net.minecraft.util.RandomSource
import net.minecraft.world.entity.player.Player
import net.minecraft.world.item.ItemStack
import net.minecraft.world.item.Rarity
import net.minecraft.world.item.TooltipFlag
import net.minecraft.world.level.Level
import ru.dbotthepony.mc.otm.core.TranslatableComponent
import ru.dbotthepony.mc.otm.core.nbt.set
import ru.dbotthepony.mc.otm.core.tagNotNull
import ru.dbotthepony.mc.otm.data.loot.IRandomizableItem
import java.util.*

class ProceduralExoPackSlotUpgradeItem : AbstractExoPackSlotUpgradeItem(defaultProperties()), IRandomizableItem {
	override fun getRarity(itemStack: ItemStack): Rarity {
		return when (slotCount(itemStack)) {
			in 0 .. 9 -> Rarity.COMMON
			in 10 .. 18 -> Rarity.UNCOMMON
			in 19 .. 27 -> Rarity.RARE
			in 28 .. 36 -> Rarity.EPIC
			else -> Rarity.EPIC
		}
	}

	override fun hasDescription(itemStack: ItemStack): Boolean {
		return false
	}

	override fun slotCount(itemStack: ItemStack): Int {
		return itemStack.tag?.getInt(SLOT_COUNT_KEY) ?: 0
	}

	override fun uuid(itemStack: ItemStack): UUID? {
		return null
	}

	override fun appendHoverText(
		p_41421_: ItemStack,
		p_41422_: Level?,
		tooltip: MutableList<Component>,
		p_41424_: TooltipFlag
	) {
		super.appendHoverText(p_41421_, p_41422_, tooltip, p_41424_)

		if (p_41421_.tag?.get(SLOT_COUNT_KEY) == null) {
			tooltip.add(TranslatableComponent("$descriptionId.description").withStyle(ChatFormatting.GRAY))
		}
	}

	override fun randomize(itemStack: ItemStack, random: RandomSource, ply: Player?, rarity: Rarity) {
		var amount = random.nextIntBetweenInclusive(6, 9)
		val luck = (ply?.luck ?: 0f) * 0.3f
		val finalLuck = (0.5f - (luck / 512f).coerceAtLeast(-0.3f).coerceAtMost(0.3f))

		if (random.nextFloat() >= finalLuck && rarity.ordinal >= Rarity.UNCOMMON.ordinal) {
			amount += random.nextIntBetweenInclusive(4, 9)

			if (random.nextFloat() >= finalLuck && rarity.ordinal >= Rarity.RARE.ordinal) {
				amount += random.nextIntBetweenInclusive(6, 9)

				if (random.nextFloat() >= finalLuck && rarity.ordinal >= Rarity.EPIC.ordinal) {
					amount += random.nextIntBetweenInclusive(8, 12)

					if (random.nextFloat() >= finalLuck) {
						amount += random.nextIntBetweenInclusive(14, 18)
					}
				}
			}
		}

		itemStack.tagNotNull[SLOT_COUNT_KEY] = amount
	}

	companion object {
		const val SLOT_COUNT_KEY = "slotCount"
		const val UUID_KEY = "uuid"
	}
}
